/**
 * Copyright (c) 2007-2010 Capgemini, 2010-2012 EBMWebsourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentUnsupportedOperationException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of a JMX component associated to a JBI component.
 * @author Christophe Deneux - Capgemini Sud
 */
public final class ComponentClientImpl extends AbstractServiceClient implements ComponentClient {

    /**
     * Action : shutdown a component
     */
    protected static final String SHUTDOWN = "shutDown";

    /**
     * Action : start a component
     */
    protected static final String START = "start";

    /**
     * Action : stop a component
     */
    protected static final String STOP = "stop";

    /**
     * Action : get current state
     */
    protected static final String GET_CURRENT_STATE = "getCurrentState";

    /**
     * Action : reload placeholders configuration
     */
    protected static final String RELOAD_PLACEHOLDERS = "reloadPlaceHolders";

    /**
     * Name of the runtime configuration MBean associated to the current component
     */
    private final ObjectName runtimeConfMbeanName;

    /**
     * Creates a new instance of {@link ComponentClient}
     * 
     * @param petalsDomain
     * @param componentName
     * @param mBeanServerConnection
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     */
    public ComponentClientImpl(final String petalsDomain, final String componentName,
            final MBeanServerConnection mBeanServerConnection) throws ComponentDoesNotExistException,
            ComponentErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final String sePattern = "*:type=engine,name=" + componentName;
            final ObjectName seObjName = new ObjectName(sePattern);
            final Set<ObjectName> seObjNames = this.mBeanServerConnection.queryNames(seObjName, null);
            if (seObjNames != null && seObjNames.size() == 1) {
                this.mbeanName = seObjNames.iterator().next();
            } else {
                final String bcPattern = "*:type=binding,name=" + componentName;
                final ObjectName bcObjName = new ObjectName(bcPattern);
                final Set<ObjectName> bcObjNames = this.mBeanServerConnection.queryNames(bcObjName, null);
                if (bcObjNames != null && bcObjNames.size() == 1) {
                    this.mbeanName = bcObjNames.iterator().next();
                } else {
                    throw new ComponentDoesNotExistException(componentName);
                }
            }

            this.runtimeConfMbeanName = ComponentClientImpl.retrieveRuntimeConfigurationMBean(componentName,
                    mBeanServerConnection);

        } catch (final MalformedObjectNameException e) {
            throw new ComponentErrorException(e);
        } catch (final IOException e) {
            throw new ComponentErrorException(e);
        }
    }

    private static ObjectName retrieveRuntimeConfigurationMBean(final String componentName,
            final MBeanServerConnection mBeanServerConnection) throws MalformedObjectNameException, IOException {

        final String runtimeConfPattern = "*:type=custom,name=runtime_configuration_" + componentName;
        final ObjectName runtimeConfObjName = new ObjectName(runtimeConfPattern);
        final Set<ObjectName> runtimeConfObjNames = mBeanServerConnection.queryNames(runtimeConfObjName, null);
        if (runtimeConfObjNames != null && runtimeConfObjNames.size() == 1) {
            return runtimeConfObjNames.iterator().next();
        } else {
            return null;
        }
    }

    /**
     * Creates a new instance of {@link ComponentClient}
     * 
     * @param componentObjectName
     * @param mBeanServerConnection
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     */
    public ComponentClientImpl(final ObjectName componentObjectName, final MBeanServerConnection mBeanServerConnection)
            throws ComponentDoesNotExistException,
            ComponentErrorException {

        super(componentObjectName.getDomain(), mBeanServerConnection);
        this.mbeanName = componentObjectName;

        this.runtimeConfMbeanName = null;
    }

    @Override
    public void start() throws ComponentErrorException {
        try {
            this.performAction(START, null, null);
        } catch (final PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        }
    }

    @Override
    public void stop() throws ComponentErrorException {
        try {
            this.performAction(STOP, null, null);
        } catch (final PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        }
    }

    @Override
    public void shutdown() throws ComponentErrorException {
        try {
            this.performAction(SHUTDOWN, null, null);
        } catch (final PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        }

    }

    @Override
    public State getCurrentState() throws ComponentErrorException {

        try {
            final String state = (String) this.performAction(GET_CURRENT_STATE, null, null);
            return State.parse(state);
        } catch (final PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentErrorException("Unexpected result type", e);
        }
    }

    @Override
    public void reloadConfiguration() throws ComponentErrorException {
        if (this.runtimeConfMbeanName != null) {
            try {
                this.performAction(this.runtimeConfMbeanName, RELOAD_PLACEHOLDERS, null, null);
            } catch (final PerformActionErrorException e) {
                throw new ComponentErrorException(e);
            }
        } else {
            throw new ComponentUnsupportedOperationException(RELOAD_PLACEHOLDERS);
        }
    }

}
