/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.topology.exception.AddContainerException;
import org.ow2.petals.clientserverapi.topology.exception.AttachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.DetachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistencyException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerNameProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoSubdomainConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.SubDomainNotFoundException;
import org.ow2.petals.clientserverapi.topology.exception.SubdomainIsNotDynamicException;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionDoesNotExistException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionRemoteErrorException;

/**
 * Client API of the JMX topology service.
 * 
 * @author Christophe Deneux - Linagora
 */
public final class TopologyServiceClientImpl extends AbstractServiceClient implements
        TopologyServiceClient {

    /**
     * Action : addContainerConfiguration
     */
    private static final String ADD_CONTAINER_CONFIGURATION = "addContainerConfiguration";

    /**
     * Action : attachContainerTo
     */
    private static final String ATTACH_CONTAINER = "attachContainerTo";

    /**
     * Action : detachContainer
     */
    private static final String DETACH_CONTAINER = "detachContainer";

    /**
     * Action : retrieve topology
     */
    public static final String RETRIEVE_TOPOLOGY = "retrieveTopology";

    /**
     * Action : get registry implementation
     */
    public static final String RETRIEVE_REGISTRY_IMPL = "retrieveRegistryImplementation";

    /**
     * Creates a new instance of {@link InstallationServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     */
    public TopologyServiceClientImpl(String petalsDomain,
            MBeanServerConnection mBeanServerConnection)
            throws TopologyServiceDoesNotExistException, TopologyServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, TopologyServiceClient.TOPOLOGY_SERVICE_MBEAN_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new TopologyServiceDoesNotExistException(objName.toString());
            }
        } catch (final MalformedObjectNameException e) {
            throw new TopologyServiceErrorException(e);
        } catch (final IOException e) {
            throw new TopologyServiceErrorException(e);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void addContainerConfiguration(final ContainerConfiguration addedContainer)
            throws NoContainerConfigurationProvidedException, NoSubdomainConfigurationProvidedException,
            SubDomainNotFoundException, SubdomainIsNotDynamicException, AddContainerException {

        final Object[] objects = new Object[] { addedContainer };
        final String[] types = new String[] { ContainerConfiguration.class.getName() };

        try {
            this.performAddContainerConfiguration(objects, types);
        } catch (final TopologyServiceErrorException e) {
            throw new AddContainerException(addedContainer.getName(), e);
        }

    }

    private void performAddContainerConfiguration(final Object[] objects, final String[] types)
            throws AddContainerException, TopologyServiceErrorException {

        try {
            this.performAction(ADD_CONTAINER_CONFIGURATION, objects, types);
        } catch (final PerformActionErrorException e) {
            if (e.getCause() instanceof AddContainerException) {
                throw (AddContainerException) e.getCause();
            } else {
                throw new TopologyServiceErrorException(e);
            }
        }
    }

    /**
     * {@inheritDoc}
     * 
     * @throws TopologyServiceErrorException
     *             An unexpected error occurs performing the JMX operation
     */
    @Override
    public Set<Map<String, String>> retrieveTopology(final String securityPassPhrase) throws PetalsException {

        final Object[] objects = new Object[] { securityPassPhrase };
        final String[] strings = new String[] { String.class.getName() };

        try {
            return (Set<Map<String, String>>) this.performAction(RETRIEVE_TOPOLOGY, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new TopologyServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new TopologyServiceErrorException("Unexpected result type", e);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * JMX invocation errors are reported as {@link AttachContainerException}.
     * </p>
     */
    @Override
    public void attachContainerTo(final ContainerConfiguration container, final String securityPassPhrase)
            throws NoContainerConfigurationProvidedException, NoContainerNameProvidedException,
            NoSubdomainConfigurationProvidedException, SubdomainIsNotDynamicException, InconsistencyException,
            AttachContainerException {

        final Object[] objects = new Object[] { container, securityPassPhrase };
        final String[] types = new String[] { ContainerConfiguration.class.getName(), String.class.getName() };

        try {
            this.performAction(ATTACH_CONTAINER, objects, types);
        } catch (final PerformActionDoesNotExistException e) {
            throw new AttachContainerException(null, container.getName(), e);
        } catch (final PerformActionRemoteErrorException e) {
            throw new AttachContainerException(null, container.getName(), e);
        } catch (final PerformActionErrorException e) {
            throw new AttachContainerException(null, container.getName(), e);
        }

    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * JMX invocation errors are reported as {@link DetachContainerException}.
     * </p>
     */
    @Override
    public void detachContainer() throws DetachContainerException {

        final Object[] objects = new Object[] {};
        final String[] types = new String[] {};

        try {
            this.performAction(DETACH_CONTAINER, objects, types);
        } catch (final PerformActionDoesNotExistException e) {
            throw new DetachContainerException(e);
        } catch (final PerformActionRemoteErrorException e) {
            throw new DetachContainerException(e);
        } catch (final PerformActionErrorException e) {
            throw new DetachContainerException(e);
        }
    }

    /**
     * {@inheritDoc}
     * <p>
     * JMX invocation errors are reported as {@link TopologyServiceErrorException}.
     * </p>
     */
    @Override
    public String retrieveRegistryImplementation() throws PetalsException {

        final Object[] objects = new Object[] {};
        final String[] types = new String[] {};

        try {
            return (String) this.performAction(RETRIEVE_REGISTRY_IMPL, objects, types);
        } catch (final PerformActionErrorException e) {
            throw new TopologyServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new TopologyServiceErrorException("Unexpected result type", e);
        }
    }
}
