/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import static org.junit.Assert.assertNotNull;

import java.util.Map;
import java.util.Set;

import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.NotCompliantMBeanException;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.topology.exception.AddContainerException;
import org.ow2.petals.clientserverapi.topology.exception.AttachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.DetachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistencyException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerNameProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoSecurityPassPhraseProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoSubdomainConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.SubDomainNotFoundException;
import org.ow2.petals.clientserverapi.topology.exception.SubdomainIsNotDynamicException;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.impl.mbean.TopologyService;
import org.ow2.petals.jmx.api.impl.mbean.TopologyServiceMBean;

/**
 * <p>
 * Unit tests of {@link TopologyServiceClientImpl} to check that JMX client interfaces are aligned with the JMX server
 * MBean part.
 * </p>
 * <p>
 * This unit tests implement {@link TopologyServiceMBean} to avoid to forget some methods. A unit test will only invoke
 * the associated method of {@link TopologyServiceMBean}
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class TopologyServiceClientImplTest extends AbstractServiceClientImpl implements TopologyServiceMBean {

    private TopologyServiceClient topologyServiceClient;

    @Before
    public void before() throws Exception {
        this.embeddedJmxSrvCon.registerTopologyService(new TopologyService());
        this.topologyServiceClient = jmxClient.getTopologyServiceClient();
    }

    /**
     * Check the interface consistency of
     * {@link TopologyServiceClientImpl#addContainerConfiguration(ContainerConfiguration)}
     */
    @Test
    public void testAddContainerConfiguration() throws InstanceAlreadyExistsException, MBeanRegistrationException,
            NotCompliantMBeanException, NoContainerConfigurationProvidedException,
            NoSubdomainConfigurationProvidedException, SubDomainNotFoundException, SubdomainIsNotDynamicException,
            AddContainerException, TopologyServiceErrorException, TopologyServiceDoesNotExistException,
            ConnectionErrorException {

        this.addContainerConfiguration(new ContainerConfiguration());

    }

    @Override
    public void addContainerConfiguration(final ContainerConfiguration addedContainer)
            throws NoContainerConfigurationProvidedException, NoSubdomainConfigurationProvidedException,
            SubDomainNotFoundException, SubdomainIsNotDynamicException, AddContainerException {

        this.topologyServiceClient.addContainerConfiguration(addedContainer);
    }

    /**
     * Check the interface consistency of {@link TopologyServiceClientImpl#retrieveTopology(String)}
     */
    @Test
    public void testRetrieveTopology() throws TopologyServiceErrorException, TopologyServiceDoesNotExistException,
            ConnectionErrorException, PetalsException, InstanceAlreadyExistsException, MBeanRegistrationException,
            NotCompliantMBeanException {

        assertNotNull(this.retrieveTopology("passphrase"));
    }

    @Override
    public Set<Map<String, String>> retrieveTopology(final String securityPassPhrase) throws PetalsException {

        return this.topologyServiceClient.retrieveTopology(securityPassPhrase);
    }

    /**
     * Check the interface consistency of
     * {@link TopologyServiceClientImpl#attachContainerTo(ContainerConfiguration, String)}
     */
    @Test
    public void testAttachContainerTo() throws TopologyServiceErrorException, TopologyServiceDoesNotExistException,
            ConnectionErrorException, PetalsException, InstanceAlreadyExistsException, MBeanRegistrationException,
            NotCompliantMBeanException {

        this.attachContainerTo(new ContainerConfiguration(), "passphrase");
    }

    @Override
    public void attachContainerTo(final ContainerConfiguration container, final String securityPassPhrase)
            throws NoContainerConfigurationProvidedException, NoContainerNameProvidedException,
            NoSubdomainConfigurationProvidedException, SubdomainIsNotDynamicException, InconsistencyException,
            NoSecurityPassPhraseProvidedException, AttachContainerException {

        this.topologyServiceClient.attachContainerTo(container, securityPassPhrase);
    }

    /**
     * Check the interface consistency of {@link TopologyServiceClientImpl#detachContainer()}
     */
    @Test
    public void testDetachContainerTo() throws SubdomainIsNotDynamicException, DetachContainerException {

        this.detachContainer();
    }

    @Override
    public void detachContainer() throws SubdomainIsNotDynamicException, DetachContainerException {
        this.topologyServiceClient.detachContainer();
    }

    /**
     * Check the interface consistency of {@link TopologyServiceClientImpl#retrieveRegistryImplementation()}
     */
    @Test
    public void testRetrieveRegistryImplementation() throws InstanceAlreadyExistsException, MBeanRegistrationException,
            NotCompliantMBeanException, TopologyServiceErrorException, TopologyServiceDoesNotExistException,
            ConnectionErrorException, PetalsException {

        this.retrieveRegistryImplementation();
    }

    @Override
    public String retrieveRegistryImplementation() throws PetalsException {

        return this.topologyServiceClient.retrieveRegistryImplementation();
    }

}
